// Copyright (c) 2014-2015 Titanium I.T. LLC. All rights reserved. For license, see "README" or "LICENSE" file.
(function() {
	"use strict";

	var ensure = require("./util/ensure.js");
	var shim = require("./util/shim.js");
	var quixote = require("./quixote.js");
	var QElement = require("./q_element.js");
	var QElementList = require("./q_element_list.js");
	var QViewport = require("./q_viewport.js");
	var QPage = require("./q_page.js");
	var async = require("../vendor/async-1.4.2.js");

	var Me = module.exports = function QFrame(frameDom) {
		ensure.signature(arguments, [Object]);
		ensure.that(frameDom.tagName === "IFRAME", "QFrame DOM element must be an iframe");

		this._domElement = frameDom;
		this._loaded = false;
		this._removed = false;
	};

	function loaded(self, width, height, src, stylesheets) {
		self._loaded = true;
		self._document = self._domElement.contentDocument;
		self._originalBody = self._document.body.innerHTML;
		self._originalWidth = width;
		self._originalHeight = height;
		self._originalSrc = src;
		self._originalStylesheets = stylesheets;
	}

	Me.create = function create(parentElement, options, callback) {
		ensure.signature(arguments, [Object, [Object, Function], [undefined, Function]]);
		if (callback === undefined) {
			callback = options;
			options = {};
		}
		var width = options.width || 2000;
		var height = options.height || 2000;
		var src = options.src;
		var stylesheets = options.stylesheet || [];
		if (!shim.Array.isArray(stylesheets)) stylesheets = [ stylesheets ];

		var err = checkUrls(src, stylesheets);
		if (err) return callback(err);

		var iframe = insertIframe(parentElement, width, height);
		shim.EventTarget.addEventListener(iframe, "load", onFrameLoad);
		setIframeContent(iframe, src);

		var frame = new Me(iframe);
		setFrameLoadCallback(frame, callback);

		return frame;

		function onFrameLoad() {
			// WORKAROUND Mobile Safari 7.0.0, Safari 6.2.0, Chrome 38.0.2125: frame is loaded synchronously
			// We force it to be asynchronous here
			setTimeout(function() {
				loaded(frame, width, height, src, stylesheets);
				loadStylesheets(frame, stylesheets, function() {
					frame._frameLoadCallback(null, frame);
				});
			}, 0);
		}
	};

	function setFrameLoadCallback(frame, callback) {
		frame._frameLoadCallback = callback;
	}

	function checkUrls(src, stylesheets) {
		if (!urlExists(src)) return error("src", src);

		for (var i = 0; i < stylesheets.length; i++) {
			var url = stylesheets[i];
			if (!urlExists(url)) return error("stylesheet", url);
		}

		return null;

		function error(name, url) {
			return new Error("404 error while loading " + name + " (" + url + ")");
		}
	}

	function urlExists(url) {
		if (url === undefined) return true;

		var http = new XMLHttpRequest();
		http.open('HEAD', url, false);
		http.send();
		return http.status !== 404;
	}

	function insertIframe(parentElement, width, height) {
		var iframe = document.createElement("iframe");
		iframe.setAttribute("width", width);
		iframe.setAttribute("height", height);
		iframe.setAttribute("frameborder", "0");    // WORKAROUND IE 8: don't include frame border in position calcs
		parentElement.appendChild(iframe);
		return iframe;
	}

	function setIframeContent(iframe, src) {
		if (src === undefined) {
			writeStandardsModeHtml(iframe);
		}	else {
			setIframeSrc(iframe, src);
		}
	}

	function setIframeSrc(iframe, src) {
		iframe.setAttribute("src", src);
	}

	function writeStandardsModeHtml(iframe) {
		var standardsMode = "<!DOCTYPE html>\n<html><head></head><body></body></html>";
		iframe.contentWindow.document.open();
		iframe.contentWindow.document.write(standardsMode);
		iframe.contentWindow.document.close();
	}

	function loadStylesheets(self, urls, callback) {
		async.each(urls, addLinkTag, callback);

		function addLinkTag(url, onLinkLoad) {
			var link = document.createElement("link");
			shim.EventTarget.addEventListener(link, "load", function(event) { onLinkLoad(null); });
			link.setAttribute("rel", "stylesheet");
			link.setAttribute("type", "text/css");
			link.setAttribute("href", url);
			shim.Document.head(self._document).appendChild(link);
		}
	}

	Me.prototype.reset = function() {
		ensure.signature(arguments, []);
		ensureUsable(this);

		this._document.body.innerHTML = this._originalBody;
		this.scroll(0, 0);
		this.resize(this._originalWidth, this._originalHeight);
	};

	Me.prototype.reload = function(callback) {
		ensure.signature(arguments, [Function]);
		ensureUsable(this);

		var frame = this;
		var iframe = this._domElement;
		var src = this._originalSrc;

		this.resize(this._originalWidth, this._originalHeight);
		setFrameLoadCallback(frame, callback);
		setIframeContent(iframe, src);
	};

	Me.prototype.toDomElement = function() {
		ensure.signature(arguments, []);
		ensureNotRemoved(this);

		return this._domElement;
	};

	Me.prototype.remove = function() {
		ensure.signature(arguments, []);
		ensureLoaded(this);
		if (this._removed) return;

		this._domElement.parentNode.removeChild(this._domElement);
		this._removed = true;
	};

	Me.prototype.viewport = function() {
		ensure.signature(arguments, []);
		ensureUsable(this);

		return new QViewport(this);
	};

	Me.prototype.page = function() {
		ensure.signature(arguments, []);
		ensureUsable(this);

		return new QPage(this);
	};

	Me.prototype.body = function() {
		ensure.signature(arguments, []);

		return this.get("body");
	};

	Me.prototype.add = function(html, nickname) {
		ensure.signature(arguments, [String, [undefined, String]]);
		if (nickname === undefined) nickname = html;
		ensureUsable(this);

		var tempElement = document.createElement("div");
		tempElement.innerHTML = shim.String.trim(html);
		ensure.that(
			tempElement.childNodes.length === 1,
			"Expected one element, but got " + tempElement.childNodes.length + " (" + html + ")"
		);

		var insertedElement = tempElement.childNodes[0];
		this._document.body.appendChild(insertedElement);
		return new QElement(insertedElement, this, nickname);
	};

	Me.prototype.get = function(selector, nickname) {
		ensure.signature(arguments, [String, [undefined, String]]);
		if (nickname === undefined) nickname = selector;
		ensureUsable(this);

		var nodes = this._document.querySelectorAll(selector);
		ensure.that(nodes.length === 1, "Expected one element to match '" + selector + "', but found " + nodes.length);
		return new QElement(nodes[0], this, nickname);
	};

	Me.prototype.getAll = function(selector, nickname) {
		ensure.signature(arguments, [String, [undefined, String]]);
		if (nickname === undefined) nickname = selector;

		return new QElementList(this._document.querySelectorAll(selector), this, nickname);
	};

	Me.prototype.scroll = function scroll(x, y) {
		ensure.signature(arguments, [Number, Number]);

		this._domElement.contentWindow.scroll(x, y);
	};

	Me.prototype.getRawScrollPosition = function getRawScrollPosition() {
		ensure.signature(arguments, []);

		return {
			x: shim.Window.pageXOffset(this._domElement.contentWindow, this._document),
			y: shim.Window.pageYOffset(this._domElement.contentWindow, this._document)
		};
	};

	Me.prototype.resize = function resize(width, height) {
		ensure.signature(arguments, [Number, Number]);

		this._domElement.setAttribute("width", "" + width);
		this._domElement.setAttribute("height", "" + height);
	};

	function ensureUsable(self) {
		ensureLoaded(self);
		ensureNotRemoved(self);
	}

	function ensureLoaded(self) {
		ensure.that(self._loaded, "QFrame not loaded: Wait for frame creation callback to execute before using frame");
	}

	function ensureNotRemoved(self) {
		ensure.that(!self._removed, "Attempted to use frame after it was removed");
	}

})();
